'use strict';

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * This script is used to inline assertions into the README.md documents.
 */
var trimCode = function trimCode(code) {
  var lines = _lodash2.default.trim(code).split('\n');

  var indendation = lines[lines.length - 1].match(/^\s+/);

  var indentSize = indendation ? indendation[0].length : 0;

  lines = _lodash2.default.map(lines, function (line, i) {
    if (i === 0) {
      return line;
    }

    return line.slice(indentSize);
  });

  return lines.join('\n');
};

var formatCodeSnippet = function formatCodeSnippet(setup) {
  var paragraphs = [];

  paragraphs.push(trimCode(setup.code));

  if (setup.settings) {
    paragraphs.push('// Settings: ' + JSON.stringify(setup.settings));
  }

  if (setup.options) {
    paragraphs.push('// Options: ' + JSON.stringify(setup.options));
  }

  if (setup.errors) {
    paragraphs.push('// Message: ' + setup.errors[0].message);
  }

  return paragraphs.join('\n');
};

var getAssertions = function getAssertions() {
  var assertionFiles = _glob2.default.sync(_path2.default.resolve(__dirname, '../test/rules/assertions/*.js'));

  var assertionNames = _lodash2.default.map(assertionFiles, function (filePath) {
    return _path2.default.basename(filePath, '.js');
  });

  var assertionCodes = _lodash2.default.map(assertionFiles, function (filePath) {
    // eslint-disable-next-line global-require, import/no-dynamic-require
    var codes = require(filePath).default;

    return {
      invalid: _lodash2.default.map(codes.invalid, formatCodeSnippet),
      valid: _lodash2.default.map(codes.valid, formatCodeSnippet)
    };
  });

  return _lodash2.default.zipObject(assertionNames, assertionCodes);
};

var updateDocuments = function updateDocuments(assertions) {
  var readmeDocumentPath = _path2.default.join(__dirname, '../README.md');

  var documentBody = _fs2.default.readFileSync(readmeDocumentPath, 'utf8');

  documentBody = documentBody.replace(/<!-- assertions ([a-z]+?) -->/ig, function (assertionsBlock) {
    var ruleName = assertionsBlock.match(/assertions ([a-z]+)/i)[1];
    var ruleAssertions = assertions[ruleName];

    if (!ruleAssertions) {
      throw new Error('No assertions available for rule "' + ruleName + '".');

      return assertionsBlock;
    }

    return 'The following patterns are considered problems:\n\n```js\n' + ruleAssertions.invalid.join('\n\n') + '\n```\n\nThe following patterns are not considered problems:\n\n```js\n' + ruleAssertions.valid.join('\n\n') + '\n```\n';
  });

  _fs2.default.writeFileSync(readmeDocumentPath, documentBody);
};

updateDocuments(getAssertions());
//# sourceMappingURL=readme-assertions.js.map